@@ -6,21 +6,23 @@ import (
 	"encoding/json"
 	"errors"
 	"fmt"
+	"maps"
 	"path/filepath"
 	"runtime"
 	"strconv"
 	"strings"
 	"time"
 
-	"github.com/containers/storage/pkg/stringid"
 	"github.com/moby/sys/capability"
 	v1 "github.com/opencontainers/image-spec/specs-go/v1"
 	rspec "github.com/opencontainers/runtime-spec/specs-go"
 	"github.com/opencontainers/runtime-tools/generate"
 	"github.com/sirupsen/logrus"
+	"go.podman.io/storage/pkg/stringid"
 	types "k8s.io/cri-api/pkg/apis/runtime/v1"
 	kubeletTypes "k8s.io/kubelet/pkg/types"
 
+	"github.com/cri-o/cri-o/internal/annotations"
 	"github.com/cri-o/cri-o/internal/config/capabilities"
 	"github.com/cri-o/cri-o/internal/config/cgmgr"
 	"github.com/cri-o/cri-o/internal/config/device"
@@ -30,7 +32,7 @@ import (
 	"github.com/cri-o/cri-o/internal/log"
 	"github.com/cri-o/cri-o/internal/oci"
 	"github.com/cri-o/cri-o/internal/storage"
-	"github.com/cri-o/cri-o/pkg/annotations"
+	v2 "github.com/cri-o/cri-o/pkg/annotations/v2"
 	"github.com/cri-o/cri-o/pkg/config"
 	"github.com/cri-o/cri-o/utils"
 )
@@ -199,7 +201,7 @@ func (c *container) SpecAddAnnotations(ctx context.Context, sb SandboxIFace, con
 	// The sandbox annotations are already filtered for the allowed
 	// annotations, there is no need to check it additionally here.
 	for k, v := range sb.Annotations() {
-		if k == annotations.OCISeccompBPFHookAnnotation+"/"+c.config.GetMetadata().GetName() {
+		if k == v2.OCISeccompBPFHook+"/"+c.config.GetMetadata().GetName() {
 			// The OCI seccomp BPF hook
 			// (https://github.com/containers/oci-seccomp-bpf-hook)
 			// uses the annotation io.containers.trace-syscall as indicator
@@ -215,11 +217,15 @@ func (c *container) SpecAddAnnotations(ctx context.Context, sb SandboxIFace, con
 			// distinguishable files.
 			log.Debugf(ctx,
 				"Annotation key for container %q rewritten to %q (value is: %q)",
-				c.config.GetMetadata().GetName(), annotations.OCISeccompBPFHookAnnotation, v,
+				c.config.GetMetadata().GetName(), v2.OCISeccompBPFHook, v,
 			)
 
-			c.config.Annotations[annotations.OCISeccompBPFHookAnnotation] = v
-			c.spec.AddAnnotation(annotations.OCISeccompBPFHookAnnotation, v)
+			if c.config.Annotations == nil {
+				c.config.Annotations = make(map[string]string)
+			}
+
+			c.config.Annotations[v2.OCISeccompBPFHook] = v
+			c.spec.AddAnnotation(v2.OCISeccompBPFHook, v)
 		} else {
 			c.spec.AddAnnotation(k, v)
 		}
@@ -249,7 +255,7 @@ func (c *container) SpecAddAnnotations(ctx context.Context, sb SandboxIFace, con
 	c.spec.AddAnnotation(annotations.SeccompProfilePath, seccompRef)
 	c.spec.AddAnnotation(annotations.Created, created.Format(time.RFC3339Nano))
 	// for retrieving the runtime path for a given platform.
-	c.spec.AddAnnotation(annotations.PlatformRuntimePath, platformRuntimePath)
+	c.spec.AddAnnotation(v2.PlatformRuntimePath, platformRuntimePath)
 
 	metadataJSON, err := json.Marshal(c.Config().GetMetadata())
 	if err != nil {
@@ -304,7 +310,7 @@ func (c *container) SpecAddAnnotations(ctx context.Context, sb SandboxIFace, con
 
 	if configStopSignal != "" {
 		// this key is defined in image-spec conversion document at https://github.com/opencontainers/image-spec/pull/492/files#diff-8aafbe2c3690162540381b8cdb157112R57
-		c.spec.AddAnnotation(annotations.StopSignalAnnotation, configStopSignal)
+		c.spec.AddAnnotation(v2.StopSignal, configStopSignal)
 	}
 
 	return nil
@@ -535,10 +541,10 @@ func (c *container) AddUnifiedResourcesFromAnnotations(annotationsMap map[string
 		return nil
 	}
 
-	annotationKey := fmt.Sprintf("%s.%s", annotations.UnifiedCgroupAnnotation, containerName)
+	annotationKey := fmt.Sprintf("%s/%s", v2.UnifiedCgroup, containerName)
 
-	annotation := annotationsMap[annotationKey]
-	if annotation == "" {
+	annotation, ok := v2.GetAnnotationValue(annotationsMap, annotationKey)
+	if !ok || annotation == "" {
 		return nil
 	}
 
@@ -554,10 +560,14 @@ func (c *container) AddUnifiedResourcesFromAnnotations(annotationsMap map[string
 		c.spec.Config.Linux.Resources.Unified = make(map[string]string)
 	}
 
-	for _, r := range strings.Split(annotation, ";") {
-		parts := strings.SplitN(r, "=", 2)
+	for entry := range strings.SplitSeq(annotation, ";") {
+		if entry == "" {
+			continue
+		}
+
+		parts := strings.SplitN(entry, "=", 2)
 		if len(parts) != 2 {
-			return fmt.Errorf("invalid annotation %q", annotations.UnifiedCgroupAnnotation)
+			return fmt.Errorf("invalid annotation %q", annotationKey)
 		}
 
 		d, err := b64.StdEncoding.DecodeString(parts[1])
@@ -895,9 +905,7 @@ func (c *container) SpecSetLinuxContainerResources(resources *types.LinuxContain
 			specgen.Config.Linux.Resources.Unified = make(map[string]string, len(resources.GetUnified()))
 		}
 
-		for key, value := range resources.GetUnified() {
-			specgen.Config.Linux.Resources.Unified[key] = value
-		}
+		maps.Copy(specgen.Config.Linux.Resources.Unified, resources.GetUnified())
 	}
 
 	return nil
