@@ -10,17 +10,18 @@ import (
 	"sync"
 	"time"
 
-	"github.com/containers/common/pkg/hooks"
-	cstorage "github.com/containers/storage"
-	"github.com/containers/storage/pkg/ioutils"
-	cmount "github.com/containers/storage/pkg/mount"
-	"github.com/containers/storage/pkg/truncindex"
-	json "github.com/json-iterator/go"
+	json "github.com/goccy/go-json"
 	rspec "github.com/opencontainers/runtime-spec/specs-go"
 	selinux "github.com/opencontainers/selinux/go-selinux"
 	"github.com/sirupsen/logrus"
+	"go.podman.io/common/pkg/hooks"
+	cstorage "go.podman.io/storage"
+	"go.podman.io/storage/pkg/ioutils"
+	cmount "go.podman.io/storage/pkg/mount"
+	"go.podman.io/storage/pkg/truncindex"
 	types "k8s.io/cri-api/pkg/apis/runtime/v1"
 
+	"github.com/cri-o/cri-o/internal/annotations"
 	"github.com/cri-o/cri-o/internal/hostport"
 	"github.com/cri-o/cri-o/internal/lib/constants"
 	"github.com/cri-o/cri-o/internal/lib/sandbox"
@@ -31,7 +32,7 @@ import (
 	"github.com/cri-o/cri-o/internal/registrar"
 	"github.com/cri-o/cri-o/internal/storage"
 	"github.com/cri-o/cri-o/internal/storage/references"
-	"github.com/cri-o/cri-o/pkg/annotations"
+	v2 "github.com/cri-o/cri-o/pkg/annotations/v2"
 	libconfig "github.com/cri-o/cri-o/pkg/config"
 )
 
@@ -42,6 +43,8 @@ const (
 
 // ContainerServer implements the ImageServer.
 type ContainerServer struct {
+	*statsserver.StatsServer
+
 	runtime              *oci.Runtime
 	store                cstorage.Store
 	storageImageServer   storage.ImageServer
@@ -51,7 +54,6 @@ type ContainerServer struct {
 	podNameIndex         *registrar.Registrar
 	podIDIndex           *truncindex.TruncIndex
 	Hooks                *hooks.Manager
-	*statsserver.StatsServer
 
 	stateLock sync.Locker
 	state     *containerServerState
@@ -205,7 +207,11 @@ func (c *ContainerServer) LoadSandbox(ctx context.Context, id string) (sb *sandb
 	}
 
 	sbox := sandbox.NewBuilder()
+
 	name := m.Annotations[annotations.Name]
+	if name == "" {
+		return nil, errors.New("sandbox name annotation cannot be empty")
+	}
 
 	name, err = c.ReservePodName(id, name)
 	if err != nil {
@@ -226,6 +232,19 @@ func (c *ContainerServer) LoadSandbox(ctx context.Context, id string) (sb *sandb
 		return nil, fmt.Errorf("error unmarshalling %s annotation: %w", annotations.Metadata, err)
 	}
 
+	// Validate critical metadata fields to prevent restoring corrupt/malicious configs
+	if metadata.GetName() == "" {
+		return nil, errors.New("sandbox metadata name cannot be empty")
+	}
+
+	if metadata.GetNamespace() == "" {
+		return nil, errors.New("sandbox metadata namespace cannot be empty")
+	}
+
+	if metadata.GetUid() == "" {
+		return nil, errors.New("sandbox metadata uid cannot be empty")
+	}
+
 	processLabel := m.Process.SelinuxLabel
 	mountLabel := m.Linux.MountLabel
 
@@ -264,16 +283,16 @@ func (c *ContainerServer) LoadSandbox(ctx context.Context, id string) (sb *sandb
 	}
 
 	podLinuxOverhead := types.LinuxContainerResources{}
-	if v, found := m.Annotations[annotations.PodLinuxOverhead]; found {
+	if v, found := v2.GetAnnotationValue(m.Annotations, v2.PodLinuxOverhead); found {
 		if err := json.Unmarshal([]byte(v), &podLinuxOverhead); err != nil {
-			return nil, fmt.Errorf("error unmarshalling %s annotation: %w", annotations.PodLinuxOverhead, err)
+			return nil, fmt.Errorf("error unmarshalling %s annotation: %w", v2.PodLinuxOverhead, err)
 		}
 	}
 
 	podLinuxResources := types.LinuxContainerResources{}
-	if v, found := m.Annotations[annotations.PodLinuxResources]; found {
+	if v, found := v2.GetAnnotationValue(m.Annotations, v2.PodLinuxResources); found {
 		if err := json.Unmarshal([]byte(v), &podLinuxResources); err != nil {
-			return nil, fmt.Errorf("error unmarshalling %s annotation: %w", annotations.PodLinuxResources, err)
+			return nil, fmt.Errorf("error unmarshalling %s annotation: %w", v2.PodLinuxResources, err)
 		}
 	}
 
@@ -287,14 +306,22 @@ func (c *ContainerServer) LoadSandbox(ctx context.Context, id string) (sb *sandb
 	sbox.SetHostname(m.Annotations[annotations.HostName])
 	sbox.SetPortMappings(portMappings)
 	sbox.SetHostNetwork(hostNetwork)
-	sbox.SetUsernsMode(m.Annotations[annotations.UsernsModeAnnotation])
+
+	usernsMode, _ := v2.GetAnnotationValue(m.Annotations, v2.UsernsMode)
+	sbox.SetUsernsMode(usernsMode)
 	sbox.SetPodLinuxOverhead(&podLinuxOverhead)
 	sbox.SetPodLinuxResources(&podLinuxResources)
 	sbox.SetHostnamePath(m.Annotations[annotations.HostnamePath])
 	sbox.SetNamespaceOptions(&nsOpts)
 	sbox.SetSeccompProfilePath(spp)
 	sbox.SetCreatedAt(created)
-	sbox.SetNamespace(m.Annotations[annotations.Namespace])
+
+	namespace := m.Annotations[annotations.Namespace]
+	if namespace == "" {
+		return nil, errors.New("sandbox namespace cannot be empty")
+	}
+
+	sbox.SetNamespace(namespace)
 	sbox.SetKubeName(m.Annotations[annotations.KubeName])
 
 	sb, err = sbox.GetSandbox()
@@ -350,12 +377,14 @@ func (c *ContainerServer) LoadSandbox(ctx context.Context, id string) (sb *sandb
 	// We should not take whether the server currently has DropInfraCtr specified, but rather
 	// whether the server used to.
 	wasSpoofed := false
-	if spoofed, ok := m.Annotations[annotations.SpoofedContainer]; ok && spoofed == "true" {
+	if spoofed, ok := v2.GetAnnotationValue(m.Annotations, v2.Spoofed); ok && spoofed == "true" {
 		wasSpoofed = true
 	}
 
 	if !wasSpoofed {
-		scontainer, err = oci.NewContainer(m.Annotations[annotations.ContainerID], cname, sandboxPath, m.Annotations[annotations.LogPath], labels, m.Annotations, kubeAnnotations, m.Annotations[annotations.UserRequestedImage], nil, nil, "", nil, id, false, false, false, sb.RuntimeHandler(), sandboxDir, created, m.Annotations[annotations.StopSignalAnnotation])
+		stopSignal, _ := v2.GetAnnotationValue(m.Annotations, v2.StopSignal)
+
+		scontainer, err = oci.NewContainer(m.Annotations[annotations.ContainerID], cname, sandboxPath, m.Annotations[annotations.LogPath], labels, m.Annotations, kubeAnnotations, m.Annotations[annotations.UserRequestedImage], nil, nil, "", nil, id, false, false, false, sb.RuntimeHandler(), sandboxDir, created, stopSignal)
 		if err != nil {
 			return sb, err
 		}
@@ -526,11 +555,16 @@ func (c *ContainerServer) LoadContainer(ctx context.Context, id string) (retErr
 		imageID = &id
 	}
 
-	platformRuntimePath, ok := m.Annotations[annotations.PlatformRuntimePath]
+	platformRuntimePath, ok := v2.GetAnnotationValue(m.Annotations, v2.PlatformRuntimePath)
 	if !ok {
 		platformRuntimePath = ""
 	}
 
+	stopSignal, ok := v2.GetAnnotationValue(m.Annotations, v2.StopSignal)
+	if !ok {
+		stopSignal = ""
+	}
+
 	kubeAnnotations := make(map[string]string)
 	if err := json.Unmarshal([]byte(m.Annotations[annotations.Annotations]), &kubeAnnotations); err != nil {
 		return err
@@ -541,7 +575,7 @@ func (c *ContainerServer) LoadContainer(ctx context.Context, id string) (retErr
 		return err
 	}
 
-	ctr, err := oci.NewContainer(id, name, containerPath, m.Annotations[annotations.LogPath], labels, m.Annotations, kubeAnnotations, userRequestedImage, someNameOfTheImage, imageID, "", &metadata, sb.ID(), tty, stdin, stdinOnce, sb.RuntimeHandler(), containerDir, created, m.Annotations[annotations.StopSignalAnnotation])
+	ctr, err := oci.NewContainer(id, name, containerPath, m.Annotations[annotations.LogPath], labels, m.Annotations, kubeAnnotations, userRequestedImage, someNameOfTheImage, imageID, "", &metadata, sb.ID(), tty, stdin, stdinOnce, sb.RuntimeHandler(), containerDir, created, stopSignal)
 	if err != nil {
 		return err
 	}
