@@ -15,5 +15,9 @@ func linuxContainerToNRI(ctr Container) *nri.LinuxContainer {
 		Resources:   lnx.GetLinuxResources(),
 		OomScoreAdj: nri.Int(lnx.GetOOMScoreAdj()),
 		CgroupsPath: lnx.GetCgroupsPath(),
+		IoPriority:  lnx.GetIOPriority(),
+		Scheduler:   lnx.GetScheduler(),
+		NetDevices:  lnx.GetNetDevices(),
+		Rdt:         lnx.GetRdt(),
 	}
 }
