@@ -16,17 +16,17 @@ import (
 	"time"
 
 	"github.com/BurntSushi/toml"
-	"github.com/containers/common/pkg/hooks"
 	conmonrsClient "github.com/containers/conmon-rs/pkg/client"
-	"github.com/containers/image/v5/pkg/sysregistriesv2"
-	"github.com/containers/image/v5/types"
-	"github.com/containers/storage"
 	cpConfig "github.com/cri-o/crio-credential-provider/pkg/config"
 	"github.com/cri-o/ocicni/pkg/ocicni"
 	"github.com/docker/go-units"
 	"github.com/opencontainers/runtime-spec/specs-go/features"
 	selinux "github.com/opencontainers/selinux/go-selinux"
 	"github.com/sirupsen/logrus"
+	"go.podman.io/common/pkg/hooks"
+	"go.podman.io/image/v5/pkg/sysregistriesv2"
+	"go.podman.io/image/v5/types"
+	"go.podman.io/storage"
 	"k8s.io/utils/cpuset"
 	"k8s.io/utils/ptr"
 	"tags.cncf.io/container-device-interface/pkg/cdi"
@@ -46,7 +46,7 @@ import (
 	"github.com/cri-o/cri-o/internal/config/ulimits"
 	"github.com/cri-o/cri-o/internal/log"
 	"github.com/cri-o/cri-o/internal/storage/references"
-	"github.com/cri-o/cri-o/pkg/annotations"
+	v2 "github.com/cri-o/cri-o/pkg/annotations/v2"
 	"github.com/cri-o/cri-o/server/metrics/collectors"
 	"github.com/cri-o/cri-o/server/useragent"
 	"github.com/cri-o/cri-o/utils"
@@ -58,6 +58,10 @@ const (
 	defaultGRPCMaxMsgSize = 80 * 1024 * 1024
 	// default minimum memory for all other runtimes.
 	defaultContainerMinMemory = 12 * 1024 * 1024 // 12 MiB
+	// defaultContainerCreateTimeout is the default timeout for container creation operations in seconds.
+	defaultContainerCreateTimeout = 240
+	// minimumContainerCreateTimeout is the minimum allowed timeout for container creation operations in seconds.
+	minimumContainerCreateTimeout = 30
 	// minimum memory for crun, the default runtime.
 	defaultContainerMinMemoryCrun = 500 * 1024 // 500 KiB
 	OCIBufSize                    = 8192
@@ -71,13 +75,38 @@ const (
 	MonitorExecCgroupContainer    = "container"
 )
 
+// When updating metrics, remember to update the document as well.
+const (
+	AllMetrics      = "all"
+	CPUMetrics      = "cpu"
+	DiskMetrics     = "disk"
+	DiskIOMetrics   = "diskIO"
+	HugetlbMetrics  = "hugetlb"
+	MemoryMetrics   = "memory"
+	NetworkMetrics  = "network"
+	OOMMetrics      = "oom"
+	ProcessMetrics  = "process"
+	SpecMetrics     = "spec"
+	PressureMetrics = "pressure"
+)
+
+var AvailableMetrics = []string{
+	AllMetrics,
+	CPUMetrics,
+	DiskMetrics,
+	DiskIOMetrics,
+	HugetlbMetrics,
+	MemoryMetrics,
+	NetworkMetrics,
+	OOMMetrics,
+	ProcessMetrics,
+	SpecMetrics,
+	PressureMetrics,
+}
+
 // Config represents the entire set of configuration values that can be set for
 // the server. This is intended to be loaded from a toml-encoded config file.
 type Config struct {
-	Comment          string
-	singleConfigPath string // Path to the single config file
-	dropInConfigDir  string // Path to the drop-in config files
-
 	RootConfig
 	APIConfig
 	RuntimeConfig
@@ -86,6 +115,11 @@ type Config struct {
 	MetricsConfig
 	TracingConfig
 	StatsConfig
+
+	Comment          string
+	singleConfigPath string // Path to the single config file
+	dropInConfigDir  string // Path to the drop-in config files
+
 	NRI           *nri.Config
 	SystemContext *types.SystemContext
 }
@@ -203,8 +237,9 @@ func (c *RootConfig) GetStore() (storage.Store, error) {
 
 // runtimeHandlerFeatures represents the supported features of the runtime.
 type runtimeHandlerFeatures struct {
-	RecursiveReadOnlyMounts bool `json:"-"` // Internal use only.
 	features.Features
+
+	RecursiveReadOnlyMounts bool `json:"-"` // Internal use only.
 }
 
 // RuntimeHandler represents each item of the "crio.runtime.runtimes" TOML
@@ -219,20 +254,22 @@ type RuntimeHandler struct {
 	// to a container running as privileged.
 	PrivilegedWithoutHostDevices bool `toml:"privileged_without_host_devices,omitempty"`
 	// AllowedAnnotations is a slice of experimental annotations that this runtime handler is allowed to process.
-	// The currently recognized values are:
-	// "io.kubernetes.cri-o.userns-mode" for configuring a user namespace for the pod.
-	// "io.kubernetes.cri-o.Devices" for configuring devices for the pod.
-	// "io.kubernetes.cri-o.ShmSize" for configuring the size of /dev/shm.
-	// "io.kubernetes.cri-o.UnifiedCgroup.$CTR_NAME" for configuring the cgroup v2 unified block for a container.
+	// The currently recognized values are (V2 format recommended, V1 format deprecated but supported):
+	// "userns-mode.crio.io" (V1: "io.kubernetes.cri-o.userns-mode") for configuring a user namespace for the pod.
+	// "devices.crio.io" (V1: "io.kubernetes.cri-o.Devices") for configuring devices for the pod.
+	// "shm-size.crio.io" (V1: "io.kubernetes.cri-o.ShmSize") for configuring the size of /dev/shm.
+	// "unified-cgroup.crio.io/$CTR_NAME" (V1: "io.kubernetes.cri-o.UnifiedCgroup.$CTR_NAME") for configuring the cgroup v2 unified block for a container.
 	// "io.containers.trace-syscall" for tracing syscalls via the OCI seccomp BPF hook.
-	// "io.kubernetes.cri-o.LinkLogs" for linking logs into the pod.
-	// "seccomp-profile.kubernetes.cri-o.io" for setting the seccomp profile for:
-	//   - a specific container by using: `seccomp-profile.kubernetes.cri-o.io/<CONTAINER_NAME>`
-	//   - a whole pod by using: `seccomp-profile.kubernetes.cri-o.io/POD`
+	// "link-logs.crio.io" (V1: "io.kubernetes.cri-o.LinkLogs") for linking logs into the pod.
+	// "seccomp-profile.crio.io" (V1: "seccomp-profile.kubernetes.cri-o.io") for setting the seccomp profile for:
+	//   - a specific container by using: `seccomp-profile.crio.io/<CONTAINER_NAME>`
+	//   - a whole pod by using: `seccomp-profile.crio.io/POD`
 	//   Note that the annotation works on containers as well as on images.
-	//   For images, the plain annotation `seccomp-profile.kubernetes.cri-o.io`
+	//   For images, the plain annotation `seccomp-profile.crio.io`
 	//   can be used without the required `/POD` suffix or a container name.
-	// "io.kubernetes.cri-o.DisableFIPS" for disabling FIPS mode for a pod within a FIPS-enabled Kubernetes cluster.
+	// "disable-fips.crio.io" (V1: "io.kubernetes.cri-o.DisableFIPS") for disabling FIPS mode for a pod within a FIPS-enabled Kubernetes cluster.
+	// Both V1 and V2 annotations are accepted; V2 takes precedence when both are present.
+	// See ANNOTATION_MIGRATION.md for the complete migration guide.
 	AllowedAnnotations []string `toml:"allowed_annotations,omitempty"`
 
 	// DisallowedAnnotations is the slice of experimental annotations that are not allowed for this handler.
@@ -297,6 +334,10 @@ type RuntimeHandler struct {
 	// If that is also set to "", the internal default seccomp profile will be applied.
 	SeccompProfile string `toml:"seccomp_profile,omitempty"`
 
+	// ContainerCreateTimeout is the timeout for container creation operations in seconds.
+	// If not set, defaults to 240 seconds.
+	ContainerCreateTimeout int64 `toml:"container_create_timeout,omitempty"`
+
 	// seccompConfig is the seccomp configuration for the handler.
 	seccompConfig *seccomp.Config
 }
@@ -617,7 +658,8 @@ type ImageConfig struct {
 	SignaturePolicyDir string `toml:"signature_policy_dir"`
 	// InsecureRegistries is a list of registries that must be contacted w/o
 	// TLS verification.
-	// Deprecated: use `insecure` in `registries.conf` instead.
+	//
+	// Deprecated: it's no longer effective. Please use `insecure` in `registries.conf` instead.
 	InsecureRegistries []string `toml:"insecure_registries"`
 	// ImageVolumes controls how volumes specified in image config are handled
 	ImageVolumes ImageVolumesType `toml:"image_volumes"`
@@ -745,7 +787,7 @@ type StatsConfig struct {
 	CollectionPeriod int `toml:"collection_period"`
 
 	// IncludedPodMetrics specifies the list of metrics to include when collecting pod metrics.
-	// If empty, all available metrics will be collected.
+	// If "all" is specified, all metrics are included. In that case, "all" should be the only element.
 	IncludedPodMetrics []string `toml:"included_pod_metrics"`
 }
 
@@ -755,6 +797,7 @@ type StatsConfig struct {
 type tomlConfig struct {
 	Crio struct {
 		RootConfig
+
 		API     struct{ APIConfig }     `toml:"api"`
 		Runtime struct{ RuntimeConfig } `toml:"runtime"`
 		Image   struct{ ImageConfig }   `toml:"image"`
@@ -1133,6 +1176,10 @@ func (c *Config) Validate(onExecution bool) error {
 		return fmt.Errorf("validating NRI config: %w", err)
 	}
 
+	if err := c.StatsConfig.Validate(); err != nil {
+		return fmt.Errorf("validating stats config: %w", err)
+	}
+
 	return nil
 }
 
@@ -1436,11 +1483,12 @@ func getDefaultMonitorGroup(isSystemd bool) string {
 
 func defaultRuntimeHandler(isSystemd bool) *RuntimeHandler {
 	return &RuntimeHandler{
-		RuntimeType: DefaultRuntimeType,
-		RuntimeRoot: DefaultRuntimeRoot,
+		RuntimeType:            DefaultRuntimeType,
+		RuntimeRoot:            DefaultRuntimeRoot,
+		ContainerCreateTimeout: defaultContainerCreateTimeout,
 		AllowedAnnotations: []string{
-			annotations.OCISeccompBPFHookAnnotation,
-			annotations.DevicesAnnotation,
+			v2.OCISeccompBPFHook,
+			v2.Devices,
 		},
 		MonitorEnv: []string{
 			"PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin",
@@ -1819,6 +1867,8 @@ func (r *RuntimeHandler) Validate(name string) error {
 		logrus.Errorf("Unable to set minimum container memory for runtime handler %q: %v", name, err)
 	}
 
+	r.ValidateContainerCreateTimeout(name)
+
 	if err := r.ValidateNoSyncLog(); err != nil {
 		return fmt.Errorf("no sync log: %w", err)
 	}
@@ -1961,6 +2011,20 @@ func (r *RuntimeHandler) ValidateContainerMinMemory(name string) error {
 	return nil
 }
 
+// ValidateContainerCreateTimeout sets the default container create timeout if not configured.
+func (r *RuntimeHandler) ValidateContainerCreateTimeout(name string) {
+	switch {
+	case r.ContainerCreateTimeout == 0:
+		r.ContainerCreateTimeout = defaultContainerCreateTimeout
+		logrus.Infof("Runtime handler %q container create timeout not set, using default: %d seconds", name, r.ContainerCreateTimeout)
+	case r.ContainerCreateTimeout < minimumContainerCreateTimeout:
+		logrus.Warnf("Runtime handler %q container create timeout (%d seconds) is less than minimum (%d seconds), setting to minimum: %d seconds", name, r.ContainerCreateTimeout, minimumContainerCreateTimeout, minimumContainerCreateTimeout)
+		r.ContainerCreateTimeout = minimumContainerCreateTimeout
+	default:
+		logrus.Infof("Runtime handler %q container create timeout set to: %d seconds", name, r.ContainerCreateTimeout)
+	}
+}
+
 // ValidateWebsocketStreaming can be used to verify if the runtime supports WebSocket streaming.
 func (r *RuntimeHandler) ValidateWebsocketStreaming(name string) error {
 	if r.RuntimeType != RuntimeTypePod {
@@ -2092,7 +2156,7 @@ func (r *RuntimeHandler) validateRuntimeSeccompProfile() error {
 
 func validateAllowedAndGenerateDisallowedAnnotations(allowed []string) (disallowed []string, _ error) {
 	disallowedMap := make(map[string]bool)
-	for _, ann := range annotations.AllAllowedAnnotations {
+	for _, ann := range v2.AllAllowedAnnotations {
 		disallowedMap[ann] = false
 	}
 
@@ -2147,3 +2211,17 @@ func (c *NetworkConfig) CNIManagerShutdown() {
 func (c *Config) SetSingleConfigPath(singleConfigPath string) {
 	c.singleConfigPath = singleConfigPath
 }
+
+func (c *StatsConfig) Validate() error {
+	for _, metrics := range c.IncludedPodMetrics {
+		if metrics == AllMetrics && len(c.IncludedPodMetrics) != 1 {
+			return errors.New("'all' should be only one element in included_pod_metrics")
+		}
+
+		if !slices.Contains(AvailableMetrics, metrics) {
+			return fmt.Errorf("invalid pod metrics %q, available metrics: %v", metrics, AvailableMetrics)
+		}
+	}
+
+	return nil
+}
