@@ -4,6 +4,7 @@ import (
 	"context"
 	"encoding/json"
 	"fmt"
+	"maps"
 
 	"github.com/containerd/nri/pkg/api"
 	nrigen "github.com/containerd/nri/pkg/runtime-tools/generate"
@@ -13,6 +14,7 @@ import (
 	cri "k8s.io/cri-api/pkg/apis/runtime/v1"
 	"tags.cncf.io/container-device-interface/pkg/cdi"
 
+	"github.com/cri-o/cri-o/internal/annotations"
 	"github.com/cri-o/cri-o/internal/config/cgmgr"
 	"github.com/cri-o/cri-o/internal/config/node"
 	"github.com/cri-o/cri-o/internal/config/rdt"
@@ -20,7 +22,6 @@ import (
 	"github.com/cri-o/cri-o/internal/log"
 	"github.com/cri-o/cri-o/internal/nri"
 	"github.com/cri-o/cri-o/internal/oci"
-	"github.com/cri-o/cri-o/pkg/annotations"
 )
 
 type nriAPI struct {
@@ -117,6 +118,10 @@ func (a *nriAPI) createContainer(ctx context.Context, specgen *generate.Generato
 		return err
 	}
 
+	if adjust == nil {
+		return nil
+	}
+
 	wrapgen := nrigen.SpecGenerator(specgen,
 		nrigen.WithAnnotationFilter(
 			func(values map[string]string) (map[string]string, error) {
@@ -482,6 +487,7 @@ func (a *nriAPI) EvictContainer(ctx context.Context, e *api.ContainerEviction) e
 
 type criPodSandbox struct {
 	*sandbox.Sandbox
+
 	spec *rspec.Spec
 	pid  int
 }
@@ -552,9 +558,7 @@ func (p *criPodSandbox) GetAnnotations() map[string]string {
 	}
 
 	anns := map[string]string{}
-	for key, value := range p.Annotations() {
-		anns[key] = value
-	}
+	maps.Copy(anns, p.Annotations())
 
 	return anns
 }
@@ -565,9 +569,7 @@ func (p *criPodSandbox) GetLabels() map[string]string {
 	}
 
 	labels := map[string]string{}
-	for key, value := range p.Labels() {
-		labels[key] = value
-	}
+	maps.Copy(labels, p.Labels())
 
 	return labels
 }
@@ -769,6 +771,46 @@ func (c *criContainer) GetCgroupsPath() string {
 	return c.GetSpec().Linux.CgroupsPath
 }
 
+func (c *criContainer) GetIOPriority() *api.LinuxIOPriority {
+	spec := c.GetSpec()
+	if spec.Process == nil {
+		return nil
+	}
+
+	return api.FromOCILinuxIOPriority(spec.Process.IOPriority)
+}
+
+func (c *criContainer) GetScheduler() *api.LinuxScheduler {
+	spec := c.GetSpec()
+	if spec.Process == nil || spec.Process.Scheduler == nil {
+		return nil
+	}
+
+	return api.FromOCILinuxScheduler(spec.Process.Scheduler)
+}
+
+func (c *criContainer) GetNetDevices() map[string]*api.LinuxNetDevice {
+	spec := c.GetSpec()
+	if spec.Linux == nil {
+		return nil
+	}
+
+	return api.FromOCILinuxNetDevices(spec.Linux.NetDevices)
+}
+
+func (c *criContainer) GetRdt() *api.LinuxRdt {
+	spec := c.GetSpec()
+	if spec.Linux == nil || spec.Linux.IntelRdt == nil {
+		return nil
+	}
+
+	return &api.LinuxRdt{
+		ClosId:           api.String(spec.Linux.IntelRdt.ClosID),
+		Schemata:         api.RepeatedString(spec.Linux.IntelRdt.Schemata),
+		EnableMonitoring: api.Bool(spec.Linux.IntelRdt.EnableMonitoring),
+	}
+}
+
 func (c *criContainer) GetSpec() *rspec.Spec {
 	if c.spec != nil {
 		return c.spec
@@ -817,9 +859,7 @@ func fromCRILinuxResources(c *cri.LinuxContainerResources) *api.LinuxResources {
 
 	if u := c.GetUnified(); len(u) != 0 {
 		r.Unified = make(map[string]string)
-		for k, v := range u {
-			r.Unified[k] = v
-		}
+		maps.Copy(r.GetUnified(), u)
 	}
 
 	return r
@@ -854,9 +894,7 @@ func toCRIResources(r *api.LinuxResources, oomScoreAdj int64) *cri.LinuxContaine
 
 	if u := r.GetUnified(); len(u) != 0 {
 		o.Unified = make(map[string]string)
-		for k, v := range u {
-			o.Unified[k] = v
-		}
+		maps.Copy(o.GetUnified(), u)
 	}
 
 	return o
